<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xalan="http://xml.apache.org/xalan">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" standalone="no" indent="yes"/>	

	<!-- 
	***************************************************************************************************************************************************************************
	Global variable definitions
	***************************************************************************************************************************************************************************
	-->

	<!-- *** Format utility variables *** -->

	<xsl:variable name="newline">
	<xsl:text>
	</xsl:text>
	</xsl:variable>	

	<!-- 
	***************************************************************************************************************************************************************************
	Extract the information about the decision nodes added in 4.2.0
	***************************************************************************************************************************************************************************
	-->

	<!-- *** XSLT starts at the root template *** -->
	<xsl:template match="/" >

		<xsl:variable name="OneDecisionNode" select="CollabTemplate/Scenarios/Scenario[*]/GUIDiagrams/GUIDiagram[*]/NodesGUI[*]/NodeGUI[*]/DecisionNode" />

		
		<xsl:choose>
			<xsl:when test="$OneDecisionNode/DecisionBranches">
				<xsl:element name="Scenarios" />
			</xsl:when>
			<!-- *** otherwise what should i do, need to format an empty element struct as expected by the next XSLT *** -->
			<xsl:otherwise>
				<xsl:call-template name="startProcessing" />												
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

	<xsl:template name="startProcessing">		

		<xsl:element name="Scenarios">

			<xsl:for-each select="CollabTemplate/Scenarios/Scenario">
				<xsl:element name="Scenario">	
					<xsl:attribute name="Name">
						<xsl:value-of select="@Name" />
					</xsl:attribute>

					<xsl:for-each select="GUIDiagrams/GUIDiagram">					
						<xsl:variable name="strDiagramName" select="substring-after(@Name, 'GUI__')" />
						<xsl:variable name="nodeModelDiagram" select="../../ModelScenario/Diagram[@Name = $strDiagramName]" />
						<xsl:element name="Diagram">
							<xsl:attribute name="GUIName">
								<xsl:value-of select="@Name" />
							</xsl:attribute>
							<xsl:attribute name="Name">
								<xsl:value-of select="$strDiagramName" />
							</xsl:attribute>

							<xsl:for-each select="NodesGUI/NodeGUI[DecisionNode]">
								<xsl:call-template name="writeDecisionInfo">
									<xsl:with-param name="nodeModelDiagram" select="$nodeModelDiagram" />
								</xsl:call-template>
							</xsl:for-each>
						<xsl:value-of select="$newline" />
						</xsl:element>
					</xsl:for-each>

					<xsl:value-of select="$newline" />
				</xsl:element>
			</xsl:for-each>					
				
			<xsl:value-of select="$newline" />
		</xsl:element>
		
	</xsl:template>

	<xsl:template name="writeDecisionInfo">
	<xsl:param name="nodeModelDiagram" />
		<xsl:element name="DecisionLinkNode">
			<xsl:variable name="nUniqID" select="NodeLinkGUI/@UniqIDRef" />

			<xsl:attribute name="UniqID">
				<xsl:value-of select="$nUniqID" /> 
			</xsl:attribute>

			<!-- Now Find the corresponding model diagram -->
			<xsl:variable name="nodeActionModelActivityElement" select="$nodeModelDiagram/DiagramBody/Nodes/*//ModelActivityElement[@UniqID = $nUniqID]" />

			<xsl:element name="Label" >
				<xsl:value-of select="$nodeActionModelActivityElement/Label" />
			</xsl:element>
			<xsl:element name="Desc" >
				<xsl:value-of select="$nodeActionModelActivityElement/ModelElement/Desc" />
			</xsl:element>

			<SourceLink>
				<xsl:variable name="nodeLink" select="$nodeModelDiagram/DiagramBody/Links/*/Link[Connect/@To = $nUniqID]" />
				<xsl:attribute name="UniqID">
					<xsl:value-of select="$nodeLink/ModelActivityElement/@UniqID" />
				</xsl:attribute>
				<xsl:attribute name="To">
					<xsl:value-of select="$nodeLink/Connect/@To" />
				</xsl:attribute>
				<xsl:attribute name="From">
					<xsl:value-of select="$nodeLink/Connect/@From" />
				</xsl:attribute>

				<xsl:element name="Label" >
					<xsl:value-of select="$nodeLink/ModelActivityElement/Label" />
				</xsl:element>
				<xsl:element name="Desc" >
					<xsl:value-of select="$nodeLink/ModelActivityElement/ModelElement/Desc" />
				</xsl:element>
				
			</SourceLink>

			<DestinationLinks>				
				<xsl:for-each select="$nodeModelDiagram/DiagramBody/Links/*/Link[Connect/@From = $nUniqID]">
					<DestLink>
						<xsl:attribute name="UniqID">
							<xsl:value-of select="ModelActivityElement/@UniqID" />
						</xsl:attribute>
						<xsl:attribute name="To">
							<xsl:value-of select="Connect/@To" />
						</xsl:attribute>
						<xsl:attribute name="From">
							<xsl:value-of select="Connect/@From" />
						</xsl:attribute>
					</DestLink>
				</xsl:for-each>
			</DestinationLinks>

		</xsl:element>		
	</xsl:template>

</xsl:stylesheet>
