<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet 
	version="1.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:xalan="http://xml.apache.org/xalan"
	exclude-result-prefixes="xalan">


	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>	
	<xsl:strip-space elements="*" />

	<xsl:template match="/">

		<!-- *** first collect the wsdl:message(s) and wsdl:portType(s) information from collaboration template *** -->
		<xsl:variable name="treePortMessages">
			<xsl:call-template name="collectInformations">
				<xsl:with-param name="nodeBusObjRefs" select="/CollabTemplate/TemplateDefinition/BusObjRefs" />
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="nodePortMessages" select="xalan:nodeset($treePortMessages)" />		

		<InterfaceInfo>
			<xsl:copy-of select="$nodePortMessages" />
		</InterfaceInfo>

	</xsl:template>	

	<xsl:template name="collectInformations">
	<xsl:param name="nodeBusObjRefs" />
		<xsl:variable name="nodeServiceCalls" select="/CollabTemplate/Scenarios/Scenario[*]/ModelScenario/Diagram[*]/DiagramBody/Nodes/OutboundServiceNode | /CollabTemplate/Scenarios/Scenario[*]/ModelScenario/Diagram[*]/DiagramBody/Nodes/InboundServiceNode" />

		<xsl:variable name="treeTriggeringVariables">
			<xsl:for-each select="xalan:distinct($nodeBusObjRefs/BusObjRef[Verbs/Verb[*]/@IsSubscribeEvent != 'Yes']/@BusObjType)">
				<xsl:element name="TriggeringVariable">
					<xsl:attribute name="BusObjType">
						<xsl:value-of select="." />
					</xsl:attribute>
					<xsl:attribute name="name">
						<xsl:choose>
							<xsl:when test="position() = 1">
								<xsl:value-of select="'triggeringBusObj'" />
							</xsl:when>						
							<xsl:otherwise>
								<xsl:value-of select="concat('triggering', ., 'BusObj')" />
							</xsl:otherwise>
						</xsl:choose>
					</xsl:attribute>
				</xsl:element>
			</xsl:for-each>
		</xsl:variable>

		<xsl:variable name="nodeTriggeringVariables" select="xalan:nodeset($treeTriggeringVariables)" />

		<xsl:for-each select="$nodeBusObjRefs/BusObjRef">
			<xsl:variable name="strPort" select="@Name" />							
			<xsl:variable name="bIsNotUsed">
				<xsl:choose>
					<xsl:when test="count($nodeServiceCalls/../*[ServiceNode/RegBusObjRefName = $strPort]) &lt;= 0">
						<xsl:value-of select="'Yes'" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="'No'" />										
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			
			<xsl:element name="tempServiceLinkType">
				<xsl:variable name="strBusObjType"  select="normalize-space(@BusObjType)" /> 
				<xsl:attribute name="name">
					<xsl:value-of select="normalize-space($strPort)" />
				</xsl:attribute>
				<xsl:attribute name="IsNotUsed">
					<xsl:value-of select="$bIsNotUsed" />
				</xsl:attribute>				
				<xsl:attribute name="BusObjType">
					<xsl:value-of select="$strBusObjType" />
				</xsl:attribute>
								
				<xsl:element name="IsRequestResponse">
					<xsl:attribute name="type">
						<xsl:choose>
							<xsl:when test="count(Verbs/Verb[@IsSubscribeEvent = 'Yes']) &gt; 0">
								<xsl:value-of select="'Yes'" />
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="'No'" />										
							</xsl:otherwise>
						</xsl:choose>
					</xsl:attribute>
					<xsl:attribute name="variable">
						<xsl:value-of select="$nodeTriggeringVariables/TriggeringVariable[@BusObjType = $strBusObjType]/@name" />
					</xsl:attribute>

					<xsl:if test="count(Verbs/Verb[@IsSubscribeEvent = 'Yes']) &gt; 0">
						<xsl:variable name="strVerbs">
							<xsl:for-each select="xalan:distinct(Verbs/Verb[@IsSubscribeEvent = 'Yes']/@Name)">
								<xsl:value-of select="normalize-space(.)" />
							</xsl:for-each>
						</xsl:variable>
						<xsl:element name="tempPortType">								
								<xsl:attribute name="name">
									<xsl:value-of select="concat($strBusObjType, $strVerbs, 'Request')" />
								</xsl:attribute>
								<xsl:element name="tempOperations">
									<xsl:for-each select="xalan:distinct(Verbs/Verb[@IsSubscribeEvent = 'Yes']/@Name)">
										<xsl:element name="tempOperation">
											<xsl:value-of select="normalize-space(.)" />
										</xsl:element>
									</xsl:for-each>
								</xsl:element>
								<xsl:element name="tempScenarios">
									<xsl:for-each select="Verbs/Verb[@IsSubscribeEvent = 'Yes']">
										<xsl:element name="tempScenarioInfo">
											<xsl:attribute name="name">
												<xsl:value-of select="normalize-space(ScenarioName)" />
											</xsl:attribute>
											<xsl:attribute name="verb">
												<xsl:value-of select="normalize-space(@Name)" />
											</xsl:attribute>																					
										</xsl:element>
									</xsl:for-each>
								</xsl:element>
						</xsl:element>
					</xsl:if>
				</xsl:element>
									
				<xsl:element name="IsOneWay">
					<xsl:call-template name="writePortTypeInfo">
						<xsl:with-param name="nodeUsingServiceCallsAs" select="$nodeServiceCalls/../InboundServiceNode[ServiceNode/RegBusObjRefName = $strPort]" />	
						<xsl:with-param name="strBusObjType" select="$strBusObjType"/>
						<xsl:with-param name="strPortName" select="normalize-space($strPort)" />	
						<xsl:with-param name="strPortType" select="'Event'" />	
					</xsl:call-template>
				</xsl:element>
									
				<xsl:element name="IsSolicitResponse">					
					<xsl:call-template name="writePortTypeInfo">
						<xsl:with-param name="nodeUsingServiceCallsAs" select="$nodeServiceCalls/../OutboundServiceNode[ @IsSync = 'true' and ServiceNode/RegBusObjRefName = $strPort]" />	
						<xsl:with-param name="nodeUsingServiceCallsForCompensation" select="$nodeServiceCalls/../OutboundServiceNode[ @IsSync = 'true' and ServiceNode/CmpBusObjRefName = $strPort]" />							
						<xsl:with-param name="strBusObjType" select="$strBusObjType"/>
						<xsl:with-param name="strPortName" select="normalize-space($strPort)" />	
						<xsl:with-param name="strPortType" select="'Request'" />	
					</xsl:call-template>
				</xsl:element>
									
				<xsl:element name="IsNotification">
					<xsl:call-template name="writePortTypeInfo">
						<xsl:with-param name="nodeUsingServiceCallsAs" select="$nodeServiceCalls/../OutboundServiceNode[ not(@IsSync = 'true') and ServiceNode/RegBusObjRefName = $strPort]" />	
						<xsl:with-param name="nodeUsingServiceCallsForCompensation" select="$nodeServiceCalls/../OutboundServiceNode[ not(@IsSync = 'true') and ServiceNode/CmpBusObjRefName = $strPort]" />	
						<xsl:with-param name="strBusObjType" select="$strBusObjType"/>
						<xsl:with-param name="strPortName" select="normalize-space($strPort)" />	
						<xsl:with-param name="strPortType" select="'Event'" />	
					</xsl:call-template>
				</xsl:element>
									
			</xsl:element>
		</xsl:for-each>		
	</xsl:template>

	<!--

	<xsl:template name="writePortTypeInfo">
	<xsl:param name="nodeUsingServiceCallsAs" />	
	<xsl:param name="strBusObjType" />
	<xsl:param name="strPortType" />	
		<xsl:attribute name="type">
			<xsl:choose>
				<xsl:when test="count($nodeUsingServiceCallsAs) &gt; 0">						
					<xsl:value-of select="'Yes'" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="'No'" />										
				</xsl:otherwise>
			</xsl:choose>
		</xsl:attribute>

		<xsl:if test="count($nodeUsingServiceCallsAs) &gt; 0">								
			<xsl:if test="count($nodeUsingServiceCallsAs[ServiceNode/RegBusObjVerb = ''])">
				<xsl:element name="tempPortType">								
					<xsl:attribute name="name">
						<xsl:value-of select="concat($strBusObjType, $strPortType)" />
					</xsl:attribute>
					<xsl:element name="tempOperations">
						<xsl:for-each select="xalan:distinct(Verbs/Verb[*]/@Name)">
							<xsl:element name="tempOperation">
								<xsl:value-of select="normalize-space(.)" />
							</xsl:element>
						</xsl:for-each>
					</xsl:element>
					<xsl:element name="tempServiceNodes">
						<xsl:for-each select="$nodeUsingServiceCallsAs[ServiceNode/RegBusObjVerb = '']">
							<xsl:element name="tempServiceNode">
								<xsl:attribute name="pd.id">
									<xsl:value-of select="normalize-space(.//ModelActivityElement/@UniqID)" />
								</xsl:attribute>
								<xsl:attribute name="verb">
									<xsl:value-of select="''" />
								</xsl:attribute>
							</xsl:element>
						</xsl:for-each>
					</xsl:element>
				</xsl:element>
			</xsl:if>

			<xsl:if test="count($nodeUsingServiceCallsAs[not(ServiceNode/RegBusObjVerb = '')])">
				<xsl:variable name="strVerbs">
					<xsl:for-each select="xalan:distinct($nodeUsingServiceCallsAs[not(ServiceNode/RegBusObjVerb = '')]/ServiceNode/RegBusObjVerb)">
						<xsl:value-of select="normalize-space(.)" />
					</xsl:for-each>
				</xsl:variable>
				<xsl:element name="tempPortType">								
					<xsl:attribute name="name">
						<xsl:value-of select="concat($strBusObjType, $strVerbs, $strPortType)" />
					</xsl:attribute>
					<xsl:element name="tempOperations">
						<xsl:for-each select="xalan:distinct($nodeUsingServiceCallsAs[not(ServiceNode/RegBusObjVerb = '')]/ServiceNode/RegBusObjVerb)">
							<xsl:element name="tempOperation">
								<xsl:value-of select="normalize-space(.)" />
							</xsl:element>
						</xsl:for-each>
					</xsl:element>
					<xsl:element name="tempServiceNodes">
						<xsl:for-each select="$nodeUsingServiceCallsAs[not(ServiceNode/RegBusObjVerb = '')]">
							<xsl:element name="tempServiceNode">
								<xsl:attribute name="pd.id">
									<xsl:value-of select="normalize-space(.//ModelActivityElement/@UniqID)" />
								</xsl:attribute>
								<xsl:attribute name="verb">
									<xsl:value-of select="ServiceNode/RegBusObjVerb" />
								</xsl:attribute>
							</xsl:element>
						</xsl:for-each>
					</xsl:element>
				</xsl:element>
			</xsl:if>
		</xsl:if>	
	</xsl:template>

	-->
	<xsl:template name="writePortTypeInfo">
	<xsl:param name="nodeUsingServiceCallsAs" />
	<xsl:param name="nodeUsingServiceCallsForCompensation" select="xalan:nodeset(null)" />			
	<xsl:param name="strBusObjType" />
	<xsl:param name="strPortName" />		
	<xsl:param name="strPortType" />		
		<xsl:attribute name="type">
			<xsl:choose>
				<xsl:when test="count($nodeUsingServiceCallsAs) &gt; 0 or count($nodeUsingServiceCallsForCompensation) &gt; 0">						
					<xsl:value-of select="'Yes'" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="'No'" />										
				</xsl:otherwise>
			</xsl:choose>
		</xsl:attribute>

		<xsl:if test="count($nodeUsingServiceCallsAs) &gt; 0 or count($nodeUsingServiceCallsForCompensation) &gt; 0">								
			<xsl:if test="count($nodeUsingServiceCallsAs[ServiceNode/RegBusObjVerb = '']) or count($nodeUsingServiceCallsForCompensation[ServiceNode/CmpBusObjVerb = ''])">
				<xsl:element name="tempPortType">								
					<xsl:attribute name="name">
						<xsl:value-of select="concat($strBusObjType, $strPortType)" />
					</xsl:attribute>
					<xsl:element name="tempOperations">
						<xsl:for-each select="xalan:distinct(Verbs/Verb[*]/@Name)">
							<xsl:element name="tempOperation">
								<xsl:value-of select="normalize-space(.)" />
							</xsl:element>
						</xsl:for-each>
					</xsl:element>
					<xsl:element name="tempServiceNodes">
						<xsl:for-each select="$nodeUsingServiceCallsAs[ServiceNode/RegBusObjVerb = ''] | $nodeUsingServiceCallsForCompensation[ServiceNode/CmpBusObjVerb = '']">
							<xsl:element name="tempServiceNode">
								<xsl:attribute name="pd.id">
									<xsl:value-of select="normalize-space(.//ModelActivityElement/@UniqID)" />
								</xsl:attribute>
								<xsl:attribute name="pd.verb">
									<xsl:value-of select="''" />
								</xsl:attribute>
								<xsl:attribute name="pd.comp.verb">
									<xsl:value-of select="''" />
								</xsl:attribute>
							</xsl:element>
						</xsl:for-each>
					</xsl:element>
				</xsl:element>
			</xsl:if>

			<xsl:if test="count($nodeUsingServiceCallsAs[not(ServiceNode/RegBusObjVerb = '')]) &gt; 0 or count($nodeUsingServiceCallsForCompensation[not(ServiceNode/RegBusObjVerb = '')]) &gt; 0">
				<xsl:variable name="strVerbs">
					<xsl:for-each select="xalan:distinct($nodeUsingServiceCallsAs[not(ServiceNode/RegBusObjVerb = '')]/ServiceNode/RegBusObjVerb | $nodeUsingServiceCallsForCompensation[not(ServiceNode/CmpBusObjVerb = '')]/ServiceNode/CmpBusObjVerb)">
						<xsl:value-of select="normalize-space(.)" />
					</xsl:for-each>
				</xsl:variable>
				<xsl:element name="tempPortType">								
					<xsl:attribute name="name">
						<xsl:value-of select="concat($strBusObjType, $strVerbs, $strPortType)" />
					</xsl:attribute>
					<xsl:element name="tempOperations">
						<xsl:for-each select="xalan:distinct($nodeUsingServiceCallsAs[not(ServiceNode/RegBusObjVerb = '')]/ServiceNode/RegBusObjVerb | $nodeUsingServiceCallsForCompensation[not(ServiceNode/CmpBusObjVerb = '')]/ServiceNode/CmpBusObjVerb)">
							<xsl:element name="tempOperation">
								<xsl:value-of select="normalize-space(.)" />
							</xsl:element>
						</xsl:for-each>
					</xsl:element>
					<xsl:element name="tempServiceNodes">
						<xsl:for-each select="$nodeUsingServiceCallsAs[not(ServiceNode/RegBusObjVerb = '')] | $nodeUsingServiceCallsForCompensation[not(ServiceNode/CmpBusObjVerb = '')]">
							<xsl:element name="tempServiceNode">
								<xsl:attribute name="pd.id">
									<xsl:value-of select="normalize-space(.//ModelActivityElement/@UniqID)" />
								</xsl:attribute>								
								<xsl:attribute name="pd.verb">
									<xsl:if test="ServiceNode/RegBusObjRefName = $strPortName">
										<xsl:value-of select="ServiceNode/RegBusObjVerb" />
									</xsl:if>
								</xsl:attribute>
								<xsl:attribute name="pd.comp.verb">
									<xsl:if test="ServiceNode/CmpBusObjRefName = $strPortName">
										<xsl:value-of select="ServiceNode/CmpBusObjVerb" />
									</xsl:if>
								</xsl:attribute>
							</xsl:element>
						</xsl:for-each>
					</xsl:element>
				</xsl:element>
			</xsl:if>
		</xsl:if>	
	</xsl:template>

<!-- *** Design Document ***

	collect informations related to a port as BPEL Servicelinks.
		1. name of the port
		2. whether the port is really used by the service call(s) or not
		3. which BO type the ports uses.

		4.Is it used for Request-Response operations
			If yes:
				a. then create BPEL port type with a name, supported operations and extra scenario information

		5.Is it used for One way operations
			If yes:				
				a. then create BPEL port type with a name, all operations and the ids of service calls  for all
					service calls "without" verbs specified
				b. also create BPEL port type with a name, supported operations and the ids of service calls  for all
					service calls "with" verbs specified
					
		5.Is it used for Solicit-Response operations
			If yes:				
				a. then create BPEL port type with a name, all operations and the ids of service calls  for all
					service calls "without" verbs specified
				b. also create BPEL port type with a name, supported operations and the ids of service calls  for all
					service calls "with" verbs specified

		5.Is it used for Notification operations
			If yes:				
				a. then create BPEL port type with a name, all operations and the ids of service calls  for all
					service calls "without" verbs specified
				b. also create BPEL port type with a name, supported operations and the ids of service calls  for all
					service calls "with" verbs specified

	 *** Design Document ***
-->
</xsl:stylesheet>
